﻿
<#PSScriptInfo

.VERSION 1.2

.GUID b608a45b-6cd0-405e-bfb2-aa11450821b5

.AUTHOR Alexey Semibratov

.COMPANYNAME 

.COPYRIGHT Alexey Semibratov

.TAGS 

.LICENSEURI 

.PROJECTURI 

.ICONURI 

.EXTERNALMODULEDEPENDENCIES 

.REQUIREDSCRIPTS 

.EXTERNALSCRIPTDEPENDENCIES 

.RELEASENOTES
Version 1.2:  Added more documentation and set of required rights. Now if the device is not found in Autopilot, but exists in Intune (by serial number), it still cleans it from AD DS and AAD
Version 1.1:  Invoke-AutopilotSync, when called too soon, error out
Version 1.0:  Original public version.

#>

<# 

.SYNOPSIS
Interactive script that helps to provision Autopilot machines. Identifies and fixes issues by removing the computer from Intune, AAD, AD and Autopilot, then adds it.

MIT LICENSE

Copyright (c) 2021 Alexey Semibratov

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


.DESCRIPTION 
Runs from OOBE screen, connects to Azure AD, Intune and optionally to AD DS, finds all objects for the serial number of the machine it is running on, then deletes it from everywhere, then adds it to Autopilot again. 
Asks for deletion of each object
 Usage: 
 - The script can work from running Windows 10, but be careful removing native Azure AD joined Intune Devices - you can lock yourself out, if you do not know local administrator's password
 - Intended usage – from OOBE (Out of Box Experience)
 - While in OOBE, hits Shift+F10
 - Powershell.exe
 - Install-Script AutopilotNuke
 - Accept all prompts
 - & 'C:\Program Files\WindowsPowerShell\Scripts\AutopilotNuke.ps1'
 - The script will:
    	Download and install all required modules (accept all prompts)
    	Show you the Serial Number of the machine
    	Prompt to connect you to Azure AD and Intune Graph
    	Ask you if you want to connect to local AD (ADDS, NT Domain) so it could delete old records from there. Enter the local FQDN (domain.com, contoso.local) of your AD Domain
    	If you entered local AD domain, it will ask you for the username and password, for the username, use <NetbiosName>\User format
    	Search in Autopilot for the serial number
    	Show you all objects in Intune and AAD related to that Serial Number
    	Ask if you want to delete in from Intune then deletes
    	Ask if you want to delete in from Autopilot then deletes
    	Loop through all AAD and AD (if it was selected) objects and ask to delete them
    	Ask if you want to add it to AP then adds

Minimum security rights needed:
•	To authorize Intune Graph, you will need global admin, but this is just one time. Ask your GA to run:
    Install-PackageProvider -Name NuGet
    Install-Module AzureAD
 	Install-Module WindowsAutopilotIntune
	Install-Module Microsoft.Graph.Intune
	Connect-AzureAD
	Connect-MSGraph 
	Accept the consent prompt
•	Custom role with the following permissions required in Intune:
	Managed devices
    	Read
    	Delete
    	Update
    	Enrollment programs
    	Create device
    	Delete device
    	Read device
    	Sync device
	Assigned to All Devices (did not try scoping it with RBAC, but should work in theory)
•	Cloud device administrator role required in Azure AD
•	AD DS rights similar to Intune Connector rights: https://docs.microsoft.com/en-us/mem/autopilot/windows-autopilot-hybrid#:~:text=The%20Intune%20Connector%20for%20your,the%20rights%20to%20create%20computers. 


#> 
[CmdletBinding()]
param(
)

Write-Host "Downloading and installing all required modules, please accept all prompts"

Install-PackageProvider -Name NuGet
Install-Module AzureAD
Install-Module WindowsAutopilotIntune
Install-Module Microsoft.Graph.Intune

$session = New-CimSession
$DomainIP = $null
$de = $null
$autopilotDevices = $null
$aadDevices = $null
$intuneDevices = $null
$localADfqdn = $null
$DomainIP = $null
$de = $null

$serial = (Get-CimInstance -CimSession $session -Class Win32_BIOS).SerialNumber


Write-Host "Will be processing device with serial number: $serial"

Write-Host "Connecting to AzureAD and Intune Graph"

Connect-AzureAD

Connect-MSGraph

Write-Host "Loading all objects. This can take a while on large tenants"
$aadDevices = Get-AzureADDevice -All $true
$intuneDevices = Get-IntuneManagedDevice -Filter "contains(operatingsystem, 'Windows')" | Get-MSGraphAllPages
$autopilotDevices = Get-AutopilotDevice | Get-MSGraphAllPages

$localADfqdn = Read-Host -Prompt 'If you want to delete this computer from AD DS, please enter the FQDN of your local active directory, otherwise, to skip AD DS deletion, hit "Enter"'
if($localADfqdn -ne "" -and $localADfqdn -ne $null)
{
    $DomainIP = (Test-Connection -ComputerName $localADfqdn -Count 1 -ErrorAction SilentlyContinue).IPV4Address.IPAddressToString
}

if($DomainIP -ne $null)
{

    Write-Host Connecting to $DomainIP
    Write-Host "Please provide the username and the password (DOMAIN\UserName)"
    $ADUserName = Read-Host -Prompt 'Username'
    $ADPassword = Read-Host -Prompt 'Password' -AsSecureString
    $ADPassword = [System.Runtime.InteropServices.Marshal]::PtrToStringAuto([System.Runtime.InteropServices.Marshal]::SecureStringToBSTR($ADPassword))
    $de = New-Object System.DirectoryServices.DirectoryEntry("LDAP://$DomainIP", $ADUserName, $ADPassword) -ErrorAction Stop
    Write-Host Connected to $de.distinguishedName   
}

# Find the objects linked to the Autopilot device

$currentAutopilotDevice = $autopilotDevices | ? {$_.serialNumber -eq $serial}

if ($currentAutopilotDevice -ne $null)
{

    Write-Verbose $currentAutopilotDevice |  Format-List -Property *
    
    $relatedIntuneDevice = $intuneDevices | ? {$_.serialNumber -eq $currentAutopilotDevice.serialNumberl -or $_.serialNumber -eq $currentAutopilotDevice.serialNumber.replace(' ','') -or $_.id -eq $currentAutopilotDevice.managedDeviceId -or $_.azureADDeviceId -eq $currentAutopilotDevice.azureActiveDirectoryDeviceId}       
       
    $relatedAadDevice = $aadDevices | ? { $_.DeviceId -eq $currentAutopilotDevice.azureActiveDirectoryDeviceId }
    
    $relatedIntuneAadDevice = $aadDevices | ? { $_.DeviceId -eq $relatedIntuneDevice.azureADDeviceId }

    # Find all the Azure AD devices with the ZTDID of this Autopilot device
    $matchedDevices = $aadDevices | ? { $_.DevicePhysicalIds -match $currentAutopilotDevice.Id }

    $allAADDevices = $aadDevices | ? { $_.DeviceId -eq $currentAutopilotDevice.azureActiveDirectoryDeviceId -or $_.DeviceId -eq $relatedIntuneDevice.azureADDeviceId -or $_.DevicePhysicalIds -match $currentAutopilotDevice.Id }

    # Display a summary for this device
    Write-Host "$($currentAutopilotDevice.SerialNumber):"
    Write-Host "  Au:AAD = $($relatedAadDevice.DisplayName) $($relatedAadDevice.DeviceTrustType) Au:Intune = $($relatedIntuneDevice.DeviceName) Intune:AAD = $($relatedIntuneAadDevice.DisplayName) $($relatedIntuneAadDevice.DeviceTrustType)"

    # If there are no devices in AAD with this ZTDID, the pre-created device was removed - that's bad
        if ($matchedDevices.Count -eq 0) {
            Write-Host "  No AAD devices found with the ZTDID" -ForegroundColor Red
    }

    
    if($relatedIntuneDevice -ne $null){

        if($Host.UI.PromptForChoice('Delete Intune Device', 'Do you want to remove ' + $relatedIntuneDevice.deviceName +' from the Intune?', @('&Yes'; '&No'), 1) -eq 0){
            Remove-IntuneManagedDevice –managedDeviceId $relatedIntuneDevice.id -ErrorAction Stop
        }

    }
    
    if($Host.UI.PromptForChoice('Delete Autopilot Device', 'Do you want to remove the device with serial number ' + $currentAutopilotDevice.serialNumber +' from the Autopilot?', @('&Yes'; '&No'), 1) -eq 0){
    

        Remove-AutopilotDevice -id $currentAutopilotDevice.id -ErrorAction Stop
        Start-Sleep -Seconds 5
        Invoke-AutopilotSync -ErrorAction SilentlyContinue
        while (Get-AutopilotDevice | Get-MSGraphAllPages | ? {$_.serialNumber -eq $serial} -ne $null){
            Start-Sleep -Seconds 5                        
       }
       Write-Host "Deleted"

    }

}

Write-Host Cleaning up AD and AAD

if($allAADDevices -eq $null){
    # this serial number was not found in Autopilot Devices, but we still want to check intune devices with this serial number and search AAD and AD DS for that one
    $relatedIntuneDevice = $intuneDevices | ? {$_.serialNumber -eq $currentAutopilotDevice.serialNumberl -or $_.serialNumber -eq $currentAutopilotDevice.serialNumber.replace(' ','')}
    $allAADDevices = $aadDevices | ? { $_.DeviceId -eq $relatedIntuneDevice.azureADDeviceId }
}

foreach($aadDevice in $allAADDevices){
    if($de -ne $null){            
        $escapedguid = “\” + ((([GUID]$aadDevice.deviceID).ToByteArray() |% {“{0:x}” -f $_}) -join ‘\’)
        $searcher = New-Object System.DirectoryServices.DirectorySearcher($de,"(&(objectCategory=Computer)(ObjectGUID=$escapedguid))")
        $obj = $searcher.FindOne()
        if ($obj -ne $null){
            $objdel = $obj.GetDirectoryEntry()
            if($Host.UI.PromptForChoice('Delete Active Directory Device', 'Do you want to remove the device with the name ' + $objdel.Name +' from AD DS?', @('&Yes'; '&No'), 1) -eq 0){
            $objdel.DeleteTree()
            }
                
        }
       
    }
    if($Host.UI.PromptForChoice('Delete Azure Active Directory Device', 'Do you want to remove the device with the name ' + $aadDevice.DisplayName +' from Azure AD?', @('&Yes'; '&No'), 1) -eq 0){
        
        Remove-AzureADDevice -ObjectId $aadDevice.ObjectID -ErrorAction SilentlyContinue
    }
    
}


# Get the hash (if available)
$devDetail = (Get-CimInstance -CimSession $session -Namespace root/cimv2/mdm/dmmap -Class MDM_DevDetail_Ext01 -Filter "InstanceID='Ext' AND ParentID='./DevDetail'")
if ($devDetail)
{
	$hash = $devDetail.DeviceHardwareData
    if($Host.UI.PromptForChoice('Add Autopilot Device', 'Do you want to add the device with serial number ' + $serial +' to Autopilot?', @('&Yes'; '&No'), 1) -eq 0){
        Add-AutopilotImportedDevice -serialNumber $serial -hardwareIdentifier $hash
        Start-Sleep -Seconds 30
        Invoke-AutopilotSync -ErrorAction SilentlyContinue
        Start-Sleep -Seconds 30
        }


}

