# derived from get-windowsautopilotinfo.ps1 version 1.5
# adapted to run in an winpe environment
# outputfile composed from serial
[CmdletBinding()]
param(
	[Parameter(Mandatory=$False,ValueFromPipeline=$True,ValueFromPipelineByPropertyName=$True,Position=0)][alias("DNSHostName","ComputerName","Computer")] [String[]] $Name = @($env:ComputerName),
	[Parameter(Mandatory=$False)] [String] $OutputFile = "", 
	[Parameter(Mandatory=$False)] [Switch] $Append = $false,
	[Parameter(Mandatory=$False)] [System.Management.Automation.PSCredential] $Credential = $null,
	[Parameter(Mandatory=$False)] [Switch] $Partner = $false,
	[Parameter(Mandatory=$False)] [Switch] $Force = $false
)

Begin
{
	# set outputfile including serial number for usage in winpe
	# target is usb drive attached
	$PC = Get-WmiObject -Class Win32_BIOS
	$Serial = $PC.SerialNumber
	$USBDisk = Get-WmiObject -Class Win32_logicaldisk -Filter "DriveType = '2'"
	$ValueIsNOTPresent = [string]::IsNullOrEmpty($USBDisk)
	If (!($ValueIsNOTPresent))
	{
		$USBDriveLetter = $USBDisk.deviceid[0]
		$OutputFile = $USBDriveLetter + "\" + $serial + "-autopilothash.csv"
	}
    Else
    {
	    $OutputFile = "C:\temp\" + $serial + "-autopilothash.csv"
    }
	$comp = $ENV:computername
	$computers = ""
}

Process
	{
		$bad = $false

		# Get the common properties.
		Write-Verbose "Checking $comp"
		#$serial = $ser

		# Get the hash (if available)
		$devDetail = (Get-WMIObject -ComputerName $comp -Namespace root/cimv2/mdm/dmmap -Class MDM_DevDetail_Ext01 -Filter "InstanceID='Ext' AND ParentID='./DevDetail'")
		if ($devDetail -and (-not $Force))
		{
			$hash = $devDetail.DeviceHardwareData
		}
		else
		{
			$bad = $true
			$hash = ""
		}

		# If the hash isn't available, get the make and model
		if ($bad -or $Force)
		{
			$cs = Get-WmiObject -ComputerName $comp -Class Win32_ComputerSystem
			$make = $cs.Manufacturer.Trim()
			$model = $cs.Model.Trim()
		}
		else
		{
			$make = ""
			$model = ""
		}

		# Getting the PKID is generally problematic for anyone other than OEMs, so let's skip it here
		$product = ""

		# Depending on the format requested, create the necessary object

			# Create a pipeline object
			$c = New-Object psobject -Property @{
				"Device Serial Number" = $serial
				"Windows Product ID" = $product
				"Hardware Hash" = $hash
			}


		# Write the object to the pipeline or array
		if ($bad)
		{
			# Report an error when the hash isn't available
			Write-Error -Message "Unable to retrieve device hardware data (hash) from computer $comp" -Category DeviceError
		}
		else
		{
			$computers += $c
		}

	}


End
{
    # write results to outputfile in comma separated format
    $outstring = "Device Serial Number,Windows Product ID,Hardware Hash"
    $outstring | Out-File -FilePath $OutputFile -Append
    $outstring = $serial + "," + $product + "," + $hash
    $outstring | Out-File -FilePath $OutputFile -Append
}
