﻿$ErrorActionPreference = 'silentlycontinue'

function Get-ScriptDirectory
{ 
	if($hostinvocation -ne $null)
	{
		Split-Path $hostinvocation.MyCommand.path
	}
	else
	{
		Split-Path $script:MyInvocation.MyCommand.Path
	}
}

# target is usb drive attached (if present)	
$PC = Get-WmiObject -Class Win32_BIOS
$Serial = $PC.SerialNumber
$USBDisk = Get-WmiObject -Class Win32_logicaldisk -Filter "DriveType = '2'"
$ValueIsNOTPresent = [string]::IsNullOrEmpty($USBDisk)
If (!($ValueIsNOTPresent))
{
    [String]$USBDriveLetter = $USBDisk.deviceid[0]
    # if drive letter does NOT contain : (dependant on os / ps version) add the colon 
    If (!($USBDriveLetter.contains(":")))
    {
        $USBDriveLetter = $USBDriveLetter + ":"
    }

    # MAKE DIRECTORY
    $OutAPDir = $USBDriveLetter + "\" + $serial + "_AutoPilotInfos"
    If (!(Test-Path $OutAPDir))
    {
        New-Item -Path $OutAPDir -ItemType Directory
    }
    $OutputAPCAB = $OutAPDir + "\" + $serial + "-MDMDIAG.cab"
	$OutputAPDiag = $OutAPDir + "\" + $serial + "-APDiagPS.txt"
}
Else
{
    # same to %temp% if no usb drive found
    # MAKE DIRECTORY
    $OutAPDir = $ENV:TEMP + "\" + $serial + "_AutoPilotInfos"
    If (!(Test-Path $OutAPDir))
    {
        New-Item -Path $OutAPDir -ItemType Directory
    }
    $OutputAPCAB = $OutAPDir + "\" + $serial + "-MDMDIAG.cab"
	$OutputAPDiag = $OutAPDir + "\" + $serial + "-APDiagPS.txt"
}

# get mdm diagnostics
$command = "C:\windows\system32\MdmDiagnosticsTool.exe -out " + $OutAPDir
write-host "starting $command ..." -ForegroundColor Yellow
Invoke-Expression $command
# same with full areas and to zip file, does NOT work stable
# $command = "C:\windows\system32\MdmDiagnosticsTool.exe -area Autopilot;DeviceEnrollment;Tpm;DeviceProvisioning -zip " + $OutAPDir + "\" + $serial + "-mdmdiag.zip"
# write-host "starting $command ..." -ForegroundColor Yellow
# Invoke-Expression $command

# copy json files
$CMD1 = "copy C:\Windows\ServiceState\wmansvc\*.* " + $OutAPDir
write-host "starting $CMD1 ..." -ForegroundColor Yellow
Invoke-Expression $CMD1

# call Get-AutopilotDiagnostics.ps1 with allsessions and write output to file
$ScriptRoot = Get-ScriptDirectory
[String]$Script2Call = $ScriptRoot + "\Get-AutopilotDiagnostics.ps1"
$Script2 = $Script2Call.replace("\\","\")
# if script is in root of usb drive
& $Script2 -AllSessions | Out-File $OutputAPDiag

write-host "Logfiles were generated successfully in $OutAPDir " -ForegroundColor Green
